"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyFilter = void 0;
const uniquifyArray_1 = require("../general/uniquifyArray");
const toArray_1 = require("../general/toArray");
const schemaModules_1 = require("../../data/schemaModules");
const checkFilterValidity_1 = require("./checkFilterValidity");
function applyFilter(paramObj) {
    const { data, filter, graph } = paramObj;
    let usedFilter = filter;
    if (!usedFilter && graph.defaultFilter) {
        usedFilter = graph.defaultFilter;
    }
    if (!Array.isArray(data) ||
        data.length === 0 ||
        !usedFilter ||
        Object.keys(usedFilter).length === 0) {
        return (0, uniquifyArray_1.uniquifyArray)(data);
    }
    (0, checkFilterValidity_1.checkFilterValidity)(usedFilter);
    let result = (0, uniquifyArray_1.uniquifyArray)(data);
    result = filterFromVocabulary(result, usedFilter.fromVocabulary, graph, false);
    result = filterFromVocabulary(result, usedFilter.fromVocabularyExclude, graph, true);
    const { createdTerms, unavailableTerms } = createTermInstances(result, graph);
    let filteredTerms = filterTermType(createdTerms, usedFilter.termType, false);
    filteredTerms = filterTermType(filteredTerms, usedFilter.termTypeExclude, true);
    filteredTerms = filterSuperseded(filteredTerms, usedFilter);
    filteredTerms = filterSchemaModule(filteredTerms, usedFilter.schemaModule, false);
    filteredTerms = filterSchemaModule(filteredTerms, usedFilter.schemaModuleExclude, true);
    const filteredTermIRIs = filteredTerms.map(t => t.getIRI("Compact"));
    const strictFilter = (iris) => iris.filter(iri => filteredTermIRIs.includes(iri));
    const nonStrictFilter = (iris) => iris.filter(iri => filteredTermIRIs.includes(iri) || unavailableTerms.includes(iri));
    if (usedFilter.strictMode === false) {
        return nonStrictFilter(result);
    }
    else {
        return strictFilter(result);
    }
}
exports.applyFilter = applyFilter;
function createTermInstances(data, graph) {
    const createdTerms = [];
    const unavailableTerms = [];
    for (const termIri of data) {
        try {
            createdTerms.push(graph.getTerm(termIri, {}));
        }
        catch (e) {
            unavailableTerms.push(termIri);
        }
    }
    return { createdTerms, unavailableTerms };
}
function filterFromVocabulary(terms, fromVocabularyFilter, graph, excludeCheck) {
    if (!fromVocabularyFilter) {
        return terms;
    }
    const context = graph.context;
    const namespaces = (0, toArray_1.toArray)(fromVocabularyFilter);
    for (let v = 0; v < namespaces.length; v++) {
        for (let vi = 0; vi < Object.keys(context).length; vi++) {
            if (context[Object.keys(context)[vi]] === namespaces[v]) {
                namespaces[v] = Object.keys(context)[vi];
                break;
            }
        }
    }
    const result = [];
    for (const termIri of terms) {
        const matchFound = namespaces.some(ns => termIri.startsWith(ns + ":"));
        if (matchFound && !excludeCheck) {
            result.push(termIri);
        }
        else if (!matchFound && excludeCheck) {
            result.push(termIri);
        }
    }
    return result;
}
function filterSuperseded(terms, filter) {
    if (filter.isSuperseded === undefined) {
        return terms;
    }
    const result = [];
    for (const term of terms) {
        if (!filter.isSuperseded && term.isSupersededBy() === null) {
            result.push(term);
        }
        else if (filter.isSuperseded && term.isSupersededBy() !== null) {
            result.push(term);
        }
    }
    return result;
}
function filterTermType(terms, termTypeFilter, excludeCheck) {
    if (termTypeFilter === undefined) {
        return terms;
    }
    const termTypeArray = (0, toArray_1.toArray)(termTypeFilter);
    const result = [];
    for (const term of terms) {
        if ((termTypeArray.includes(term.getTermTypeLabel()) && !excludeCheck) ||
            (!termTypeArray.includes(term.getTermTypeLabel()) && excludeCheck)) {
            result.push(term);
        }
    }
    return result;
}
function filterSchemaModule(terms, schemaModuleFilter, excludeCheck) {
    if (schemaModuleFilter === undefined) {
        return terms;
    }
    const schemaModuleArray = (0, toArray_1.toArray)(schemaModuleFilter);
    const result = [];
    for (const term of terms) {
        const schemaModuleMatch = (0, schemaModules_1.getSchemaModuleMatch)(term.getVocabulary());
        if (!schemaModuleMatch ||
            (schemaModuleArray.includes(schemaModuleMatch) && !excludeCheck) ||
            (!schemaModuleArray.includes(schemaModuleMatch) && excludeCheck)) {
            result.push(term);
        }
    }
    return result;
}
//# sourceMappingURL=applyFilter.js.map